

#define DI_PULS 2  //Digital IN: Spannung der Signalleitung (Gefiltert über 47K/10n)
#define DI_DOOR 3  //Digital IN: Türkontakt. Schaltet gegen Masse; Via 10K Pullup an 5V. 
                   //            Über 47K an Eingang. Vorsichtshalber: 100nF am Eingang gegen GND
                   // Die beiden 47K und die 100nF (bzw. 10nF) müssen direkt am Eingang sitzen. Die sind wichtig, um beim
                   //Pegelwechsel die zwangsläufig entstehenden Spannungsspitzen durch die Leitungsinduktivität     
                   //auf einen ungefährlichen Strom für die Schutzdioden des Ports zu reduzieren.
#define DO_PULS 4  //Digital out: via R-C-R (270R, 100nF, 8K2) an Transistor BC639, der die Signalleitung auf Masse zieht
#define DO_NEO  5  //Neopixel-Data. Direkt am 1. Neopixel über 470 Ohm an den Din Pin des Pixels.

#define MAX_ADRESSE   30
#define ZYKLUSZEIT     1 //[ms]
#define PULSZEIT_KURZ 20 //[ms]
#define PULSZEIT_LANG 35 //[ms]
#define PULSABSTAND   50 //[ms] Abstand der steigenden Flanken
#define LANGE_PAUSE  130 //[ms] deutlich länger als in den Slaves


/*
 * Hinweis:
 *   Der DO_PULS Ausgang wird bewusst nie auf Low geschaltet.
 *   Dadurch kann der Transistor relativ schnell eingeschaltet werden (niederohmiges aufladen des 1uF Elkos vor der Basis),
 *   abschalten wird er sich aber recht langsam durch leeren des Elkos mit dem Basisstrom.
 *   Würde man den Port auf LOW schalten, würde sich der Elko auch über den Port entladen.
 *   Der Widerstand zwischen 1uF Elko und Basis des Transistors ist nötig, damit die Elkospannung über 0.7V steigen kann.
 *   Der 270 Ohm Widerstand vor dem Elko reduziert den Ausgangsstrom des Ports auf ca. 20mA.
 *   Eventuell ist es nötig, zwischen Basis und Emitter einen 47K Widerstand zu setzen, damit der Transistor sich nichts aus der Luft einfängt.
 */

#include <Adafruit_NeoPixel.h>
// Parameter 1 = number of pixels in strip
// Parameter 2 = Arduino pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
//   NEO_RGBW    Pixels are wired for RGBW bitstream (NeoPixel RGBW products)
Adafruit_NeoPixel strip = Adafruit_NeoPixel(MAX_ADRESSE, DO_NEO, NEO_GRB + NEO_KHZ800);


unsigned long t;

//--------------------------------------------------------------------------
void pulsON(void){   //Ausgang auf 5V
//--------------------------------------------------------------------------
   pinMode(DO_PULS, OUTPUT);  
   digitalWrite(DO_PULS, HIGH);
}

//--------------------------------------------------------------------------
void pulsOFF(void){ //Ausgang hochohmig
//--------------------------------------------------------------------------
   pinMode(DO_PULS, INPUT);  
   digitalWrite(DO_PULS, LOW); //Pullup aus
}

//--------------------------------------------------------------------------
void setup() {
//--------------------------------------------------------------------------
    pinMode(DO_PULS, OUTPUT);  
    pinMode(DI_DOOR, INPUT);  
    pinMode(DI_PULS, INPUT);  
    strip.begin();
    strip.show(); // Initialize all pixels to 'off'    
    t = millis();
}
//--------------------------------------------------------------------------
void loop() {
//--------------------------------------------------------------------------

static int timer_ms=0;
static int duration_ms=0;
static byte status=0;
static int pulsLength[MAX_ADRESSE];
static byte doorStatus[MAX_ADRESSE]; //0=closed 1=open, 2=unknown
static byte index;

    if ((millis() - t) >= ZYKLUSZEIT) { 
        t += ZYKLUSZEIT;
        
        switch(status) {
        case 0: //Erzeuge Startpuls
            pulsON();
            timer_ms = PULSZEIT_KURZ;
            status++;
            break;
        case 1: //Warte, bis mein Puls zuende    
            timer_ms -= ZYKLUSZEIT;
            if (timer_ms <=0) {
                pulsOFF();
                timer_ms=PULSABSTAND-PULSZEIT_KURZ; //Pausenzeit
                status++;
            }
            break;
         case 2: //Warte auf Ende der Pause 
            timer_ms -= ZYKLUSZEIT;
            if (timer_ms <=0) {
                timer_ms = 0;
                duration_ms=PULSABSTAND;
                for (byte i=0; i < MAX_ADRESSE; i++) pulsLength[i]=0;
                status++;
            }
            break;
         case 3: //erfasse Pulslängen
            duration_ms += ZYKLUSZEIT;
            index =  duration_ms / PULSABSTAND;
            if (index < MAX_ADRESSE) {
                if (!digitalRead(DI_PULS)) {
                    pulsLength[index]+=ZYKLUSZEIT;
                }
            }
            if (duration_ms > MAX_ADRESSE * PULSABSTAND) {
                timer_ms = 0;
                status++; 
            }
            break;
         case 4: //Auswertung   
            timer_ms += ZYKLUSZEIT;
            for (byte i=0; i < MAX_ADRESSE; i++) {
              if (pulsLength[i] < PULSZEIT_KURZ / 2) { //Kein Puls
                doorStatus[i] = 2;
              } else 
              if (pulsLength[i] <= (PULSZEIT_KURZ + PULSZEIT_LANG) / 2) {  //Kurzer Puls
                doorStatus[i] = 0;
              } else {
                doorStatus[i] = 1;
              }
            }
            status++;
            break;
         case 5: //Anzeige
            //Was in diesem case  gemacht wird, darf ruhig laenger als eine Zykluszeit dauern, jedoch nicht länger als die lange Pause.
            //Da zu t immer nur die Zykluszeit aufaddiert wird, wird ein eventuelles Hinterherhinken in Status 6 wieder aufgeholt, 
            //indem dort die Zykluszeit nahe null sein wird, bis t wieder millis() entspricht.

            //Status auf Neopixel-LEDs ausgeben
            for (byte i=0; i < MAX_ADRESSE; i++) {
              switch(doorStatus[i]) {
                case 0: strip.setPixelColor(i, strip.Color(255,   0, 0)); break; //rot
                case 1: strip.setPixelColor(i, strip.Color(  0, 255, 0)); break; //gruen
                case 2: strip.setPixelColor(i, strip.Color(255, 255, 0)); break; //gelb
              }
            }
            strip.show();
            timer_ms += ZYKLUSZEIT;
            status++;
            break;
         case 6: //Warte auf Ende der langen Pause   
            timer_ms += ZYKLUSZEIT;
            if (timer_ms >= LANGE_PAUSE) status=0;
            break;
        default: status=0;    
        }//switch
    }//Zyklus
}
